/*
 * Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package monasca.common.model.metric;

import java.util.Map;

import com.google.common.base.Preconditions;

/**
 * A metric envelope.
 */
public class MetricEnvelope {
  public Metric metric;
  public Map<String, Object> meta;
  public long creationTime;

  protected MetricEnvelope() {
  }

  public MetricEnvelope(Metric metric) {
    Preconditions.checkNotNull(metric, "metric");
    this.metric = metric;
    this.creationTime = System.currentTimeMillis()/1000;
  }

  public MetricEnvelope(Metric metric, Map<String, Object> meta) {
    this(metric);
    Preconditions.checkNotNull(meta, "meta");
    this.meta = meta;
  }

  @Override
  public String toString() {
    return "MetricEnvelope{" +
           "metric=" + metric +
           ", meta=" + meta +
           ", creationTime=" + creationTime +
           '}';
  }
}
