# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# __title__ = 'kafka'
from .version import __version__
__author__ = 'David Arthur'
__license__ = 'Apache License 2.0'
__copyright__ = 'Copyright 2015, David Arthur under Apache License, v2.0'
__version__ = __version__

# from monasca_common.kafka_lib.client import KafkaClient
# from monasca_common.kafka_lib.conn import KafkaConnection
# from monasca_common.kafka_lib.protocol import (
#     create_message, create_gzip_message, create_snappy_message
# )
# from monasca_common.kafka_lib.producer import SimpleProducer, KeyedProducer
# from monasca_common.kafka_lib.partitioner import RoundRobinPartitioner,
#  HashedPartitioner, Murmur2Partitioner
# from monasca_common.kafka_lib.consumer import SimpleConsumer, MultiProcessConsumer, KafkaConsumer
#
# __all__ = [
#     'KafkaClient', 'KafkaConnection', 'SimpleProducer', 'KeyedProducer',
#     'RoundRobinPartitioner', 'HashedPartitioner', 'SimpleConsumer',
#     'MultiProcessConsumer', 'create_message', 'create_gzip_message',
#     'create_snappy_message', 'KafkaConsumer',
# ]
